#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
Auditor has one task: inspection of metadata.json and PPR.xml for required properties
"""
import json
import logging
import os.path
import shutil
from pathlib import Path
from typing import Dict, List

import prettierfier
from bs4 import BeautifulSoup
from casa_envoy.interfaces import AuditorIF
from casa_envoy.schema import AbstractTextFile


def get_fields_for(product_type: str, filename: str) -> list:
    cal_metadata_list = [
        "fileSetIds",
        "workflowName",
        "systemId",
        "creationTime",
        "productLocator",
        "destinationDirectory",
    ]
    img_metadata_list = [
        "fileSetIds",
        "workflowName",
        "systemId",
        "creationTime",
        "productLocator",
        "destinationDirectory",
        "calibrationSourceDirectory",
        "cmsName",
    ]

    restore_metadata_list = [
        "calProductLocator",
    ]

    ppr_list = [
        "RootDirectory",
        "RelativePath",
        "SdmIdentifier",
    ]

    if ".xml" in filename:
        return ppr_list

    if ".json" in filename and "cal" in product_type or product_type == "integrated":
        return cal_metadata_list
    elif ".json" in filename and "img" in product_type:
        return img_metadata_list
    elif ".json" in filename and product_type == "restore":
        return cal_metadata_list + restore_metadata_list
    elif ".json" in filename and product_type == "seci":
        return []


def get_xml_content(file: AbstractTextFile):
    return BeautifulSoup(file.content, "xml")


def get_value_for(file: AbstractTextFile, key: str) -> str:
    if ".xml" in file.filename:
        ppr_content = get_xml_content(file)
        return ppr_content.find(key).string

    if ".json" in file.filename:
        content = json.loads(file.content)
        return content[key]


class AuditFiles(AuditorIF):
    def __init__(self, files: List[AbstractTextFile], settings: Dict[str, str]):
        self.logger = logging.getLogger("casa_envoy")
        self.files = files
        self.settings = settings
        self.product_type = settings.get("product_type")

    def check_required_fields(self, file: AbstractTextFile, fields: list) -> bool:
        missing = []
        content = file.content
        if ".xml" in file.filename:
            ppr_content = BeautifulSoup(content, "xml")
            for tag in fields:
                try:
                    ppr_content.find(tag).string
                except:
                    missing.append(tag)

        if ".json" in file.filename:
            for tag in fields:
                if tag not in content:
                    missing.append(tag)
        if len(missing) > 0:
            print(f"Missing fields: {missing}")
            return False
        else:
            return True

    def correct_for_condor(self, ppr: AbstractTextFile) -> AbstractTextFile:
        shutil.copy(ppr.filename, "./working")
        os.chdir("./working")

        parsed_xml = get_xml_content(ppr)
        parsed_xml.find("RootDirectory").string = self.settings["rootDirectory"]
        parsed_xml.find("RelativePath").string = self.settings["processingDirectory"]

        with open(ppr.filename, "w") as file:
            file.write(prettierfier.prettify_xml(parsed_xml.prettify(), indent=4))

        os.chdir("../")
        return ppr

    def audit(self) -> bool:
        invalid_files = []
        for file in self.files:
            self.logger.info(f"Auditing file {file.filename}...")

            if file.filename == "PPR.xml":
                self.logger.info("Correcting PPR.xml for condor processing...")
                file = self.correct_for_condor(file)

            valid = self.check_required_fields(file=file, fields=get_fields_for(self.product_type, file.filename))
            if not valid:
                invalid_files.append(file.filename)

        if len(invalid_files) != 0:
            self.logger.error(f"INVALID FILE FOUND: {invalid_files}")
            return False
        else:
            return True


class AuditDirectories(AuditorIF):
    def __init__(self, ppr: AbstractTextFile, settings: Dict[str, str]):
        self.logger = logging.getLogger("casa_envoy")
        self.parameters = settings
        self.rootDirectory = settings["rootDirectory"]
        self.relative_path = settings["processingDirectory"]
        self.sdmId = get_value_for(ppr, "SdmIdentifier")

    @property
    def audit(self) -> bool:
        current = os.getcwd()
        needed = self.rootDirectory + "/" + self.relative_path
        if needed != current:
            self.logger.error("DIRECTORY ERROR: not in correct directory for processing.")
            return False
        else:
            working = Path(current + "/working").is_dir()
            products = Path(current + "/products").is_dir()
            rawdata = Path(current + "/rawdata").is_dir()

            if working and products and rawdata:
                self.logger.info("Checking for rawdata...")
                data = os.listdir(Path(current + "/rawdata/"))
                if len(data) > 0:
                    self.logger.info("Data is available. Proceeding...")

                    if self.parameters["product_type"] == "restore":
                        self.logger.info("Checking products/ for calibration tables...")
                        cal_data = os.listdir(Path(current + "/products/"))
                        if len(cal_data) > 0:
                            self.logger.info("Calibration data is available. Proceeding...")
                        else:
                            self.logger.error("FAILURE: calibration data not found in products/")
                            return False

                    return True
                else:
                    self.logger.error("FAILURE: data not found in rawdata/")
                    return False
            else:
                self.logger.error("DIRECTORY ERROR: A directory is missing from the processing root directory.")
