#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
# -------------------------------------------------------------------------
#
#        P R O D U C T   F I N D I N G
#
# -------------------------------------------------------------------------
import abc
import os
import pathlib
from typing import Iterator

from .products import CalibratedMS, ExecutionBlock, Image, SpooledProduct, Weblog


class ProductFinder(abc.ABC):
    """
    Locates products for the delivery to deliver
    """

    @abc.abstractmethod
    def find_products(self) -> Iterator[SpooledProduct]:
        """
        Find products and return an iterator of them

        :return: Iterator of SpooledProducts
        """
        pass


# TODO: Implement after the initial delivery sketch is up and running
# class PiperesultsProductFinder(ProductFinder):
#     def __init__(self, path: pathlib.Path):
#         self.path = path
#
#     def find_products(self) -> Iterator[SpooledProduct]:
#         raise NotImplementedError


class HeuristicProductFinder(ProductFinder):
    def __init__(self, path: pathlib.Path):
        self.path = path

    def find_products(self) -> Iterator[SpooledProduct]:
        """
        Identify the product type and the number of products and returns them as a list

        :return: List of products
        """
        for item in self.path.glob("*"):
            if item.is_dir() and item.name.endswith(".ms"):
                yield CalibratedMS(item)
            elif item.is_dir() and not item.name.startswith("tmp"):
                yield ExecutionBlock(item)
            elif item.is_file() and item.name.endswith(".fits"):
                yield Image(item)
            elif item.is_file() and item.name.startswith("weblog"):
                yield Weblog(item)
